class BaseRiSDKException(Exception):
    pass


class MethodCallError(BaseRiSDKException):
    def __init__(
        self,
        error_code: int,
        error_message: bytes,
        method_name: str,
    ) -> None:
        stripped_error_message = self.stripped_bytes(error_message)
        super().__init__(error_code, stripped_error_message, method_name)
        self.original_error_message = error_message
        self.error_message = stripped_error_message
        self.error_code = error_code
        self.method_name = method_name

    @classmethod
    def stripped_bytes(cls, error: bytes) -> bytes:
        """
        Читаем байты до первого нулевого.
        Изначально строка задаётся длиной 1000 символов.
        Текст ошибки займёт 100-200-300 символов. Остальное -- нули.

        Читаем до первого нуля, остальное нас не интересует.

        :param error:
        :return: Bytes until zero
        """
        for idx, byte in enumerate(error):
            if byte == 0:
                return error[:idx]
        return error

    @property
    def message_as_text(self) -> str:
        return self.error_message.decode()



import dataclasses


@dataclasses.dataclass(frozen=True)
class SensorVoltageSensorExtendToModelResult:
    # Код ошибки. При успехе всегда 0
    error_code: int
    # descriptor - Указатель на компонент датчика тока, напряжения и
    #    мощности конкретной модели, который получится в результате
    #    расширения
    descriptor: int



import dataclasses


@dataclasses.dataclass(frozen=True)
class ConnectorI2cReadByteResult:
    # Код ошибки. При успехе всегда 0
    error_code: int
    # value - Указатель на байт, в который запишется прочитанный байт
    value: bytes



__all__ = (
    "__version__",
    "__sdk_version__",
    "RoboIntellectSDK",
    "types",
    "utils",
    "exceptions",
)

from ri_sdk import exceptions, types, utils
from ri_sdk.__about__ import __sdk_version__, __version__
from ri_sdk.sdk_wrapper import RoboIntellectSDK



import dataclasses


@dataclasses.dataclass(frozen=True)
class ConnectorI2cWriteBytesResult:
    # Код ошибки. При успехе всегда 0
    error_code: int
    # wroteBytesLen - Указатель на количество записаных байтов
    wrote_bytes_len: int



import dataclasses


@dataclasses.dataclass(frozen=True)
class SigmodPWMExtendResult:
    # Код ошибки. При успехе всегда 0
    error_code: int
    # descriptor - Указатель на компонент ШИМ, который получится в
    #    результате расширения
    descriptor: int



import dataclasses


@dataclasses.dataclass(frozen=True)
class SensorVoltageSensorSenseResult:
    # Код ошибки. При успехе всегда 0
    error_code: int
    # voltage - Указатель на значение напряжения в цепи (Вольт)
    voltage: float
    # voltageShunt - Указатель на значение напряжения на шунте
    #    (Вольт)
    voltage_shunt: float
    # current - Указатель на значение силы тока (Ампер)
    current: float
    # power - Указатель на значение мощности (Ватт)
    power: float



import dataclasses


@dataclasses.dataclass(frozen=True)
class ExecRGBLEDGetColorResult:
    # Код ошибки. При успехе всегда 0
    error_code: int
    # r - Указатель на значение яркости для красного цвета (0 -255)
    r: int
    # g - Указатель на значение яркости для зеленого цвета (0 -255)
    g: int
    # b - Указатель на значение яркости для синего цвета (0 -255)
    b: int



import dataclasses


@dataclasses.dataclass(frozen=True)
class SigmodPWMReadByteResult:
    # Код ошибки. При успехе всегда 0
    error_code: int
    # value - Указатель на байт для чтения
    value: bytes



"""
Autogenerated!
Do not edit manually.
"""

__all__ = (
    "ConnectorExtendResult",
    "ConnectorI2cCloseResult",
    "ConnectorI2cCloseAllResult",
    "ConnectorI2cExtendResult",
    "ConnectorI2cExtendToModelResult",
    "ConnectorI2cOpenResult",
    "ConnectorI2cReadByteResult",
    "ConnectorI2cReadBytesResult",
    "ConnectorI2cSetBusResult",
    "ConnectorI2cStateResult",
    "ConnectorI2cWriteByteResult",
    "ConnectorI2cWriteBytesResult",
    "CreateBasicResult",
    "CreateDeviceComponentResult",
    "CreateGroupComponentResult",
    "CreateModelComponentResult",
    "DestroyComponentResult",
    "DestroySDKResult",
    "DeviceModelListResult",
    "ExecRServoDriveCustomDeviceInitResult",
    "ExecRServoDriveExtendResult",
    "ExecRServoDriveExtendToModelResult",
    "ExecRServoDriveGetStateResult",
    "ExecRServoDriveRotateByPulseResult",
    "ExecRServoDriveRotateByPulseOverTimeResult",
    "ExecRServoDriveRotateWithRelativeSpeedResult",
    "ExecRServoDriveRotateWithRelativeSpeedOverTimeResult",
    "ExecRServoDriveStopResult",
    "ExecRGBLEDExtendResult",
    "ExecRGBLEDExtendToModelResult",
    "ExecRGBLEDFlashingWithFrequencyResult",
    "ExecRGBLEDFlashingWithPauseResult",
    "ExecRGBLEDFlickerResult",
    "ExecRGBLEDGetColorResult",
    "ExecRGBLEDGetStateResult",
    "ExecRGBLEDSinglePulseResult",
    "ExecRGBLEDStopResult",
    "ExecServoDriveCustomDeviceInitResult",
    "ExecServoDriveExtendResult",
    "ExecServoDriveExtendToModelResult",
    "ExecServoDriveGetCurrentAngleResult",
    "ExecServoDriveGetStateResult",
    "ExecServoDriveMinStepRotateResult",
    "ExecServoDriveRotateResult",
    "ExecServoDriveRotateWithRelativeSpeedResult",
    "ExecServoDriveSetPositionToMidWorkingRangeResult",
    "ExecServoDriveStopResult",
    "ExecServoDriveTurnResult",
    "ExecServoDriveTurnByDutyCycleResult",
    "ExecServoDriveTurnByPulseResult",
    "ExecServoDriveTurnWithRelativeSpeedResult",
    "ExecutorExtendResult",
    "ExecutorStateResult",
    "InitSDKResult",
    "LinkLedToControllerResult",
    "LinkPWMToControllerResult",
    "LinkRServodriveToControllerResult",
    "LinkServodriveToControllerResult",
    "LinkVoltageSensorToControllerResult",
    "SensorExtendResult",
    "SensorVoltageSensorCurrentResult",
    "SensorVoltageSensorCustomDeviceInitResult",
    "SensorVoltageSensorExtendResult",
    "SensorVoltageSensorExtendToModelResult",
    "SensorVoltageSensorPowerResult",
    "SensorVoltageSensorReadRegBytesResult",
    "SensorVoltageSensorSenseResult",
    "SensorVoltageSensorVoltageResult",
    "SensorVoltageSensorVoltageShuntResult",
    "SensorVoltageSensorWriteRegBytesResult",
    "SigmodPWMCloseResult",
    "SigmodPWMExtendResult",
    "SigmodPWMExtendToModelResult",
    "SigmodPWMGetFreqResult",
    "SigmodPWMGetPortDutyCycleResult",
    "SigmodPWMGetPortFreqResult",
    "SigmodPWMGetResolutionResult",
    "SigmodPWMReadByteResult",
    "SigmodPWMReadRegBytesResult",
    "SigmodPWMResetAllResult",
    "SigmodPWMResetPortResult",
    "SigmodPWMSetFreqResult",
    "SigmodPWMSetPortDutyCycleResult",
    "SigmodPWMSetPortFreqResult",
    "SigmodPWMWriteByteResult",
    "SigmodPWMWriteRegBytesResult",
)

from .connector_extend_result import ConnectorExtendResult
from .connector_i2c_close_all_result import ConnectorI2cCloseAllResult
from .connector_i2c_close_result import ConnectorI2cCloseResult
from .connector_i2c_extend_result import ConnectorI2cExtendResult
from .connector_i2c_extend_to_model_result import ConnectorI2cExtendToModelResult
from .connector_i2c_open_result import ConnectorI2cOpenResult
from .connector_i2c_read_byte_result import ConnectorI2cReadByteResult
from .connector_i2c_read_bytes_result import ConnectorI2cReadBytesResult
from .connector_i2c_set_bus_result import ConnectorI2cSetBusResult
from .connector_i2c_state_result import ConnectorI2cStateResult
from .connector_i2c_write_byte_result import ConnectorI2cWriteByteResult
from .connector_i2c_write_bytes_result import ConnectorI2cWriteBytesResult
from .create_basic_result import CreateBasicResult
from .create_device_component_result import CreateDeviceComponentResult
from .create_group_component_result import CreateGroupComponentResult
from .create_model_component_result import CreateModelComponentResult
from .destroy_component_result import DestroyComponentResult
from .destroy_sdk_result import DestroySDKResult
from .device_model_list_result import DeviceModelListResult
from .exec_r_servo_drive_custom_device_init_result import (
    ExecRServoDriveCustomDeviceInitResult,
)
from .exec_r_servo_drive_extend_result import ExecRServoDriveExtendResult
from .exec_r_servo_drive_extend_to_model_result import ExecRServoDriveExtendToModelResult
from .exec_r_servo_drive_get_state_result import ExecRServoDriveGetStateResult
from .exec_r_servo_drive_rotate_by_pulse_over_time_result import (
    ExecRServoDriveRotateByPulseOverTimeResult,
)
from .exec_r_servo_drive_rotate_by_pulse_result import ExecRServoDriveRotateByPulseResult
from .exec_r_servo_drive_rotate_with_relative_speed_over_time_result import (
    ExecRServoDriveRotateWithRelativeSpeedOverTimeResult,
)
from .exec_r_servo_drive_rotate_with_relative_speed_result import (
    ExecRServoDriveRotateWithRelativeSpeedResult,
)
from .exec_r_servo_drive_stop_result import ExecRServoDriveStopResult
from .exec_rgb_led_extend_result import ExecRGBLEDExtendResult
from .exec_rgb_led_extend_to_model_result import ExecRGBLEDExtendToModelResult
from .exec_rgb_led_flashing_with_frequency_result import (
    ExecRGBLEDFlashingWithFrequencyResult,
)
from .exec_rgb_led_flashing_with_pause_result import ExecRGBLEDFlashingWithPauseResult
from .exec_rgb_led_flicker_result import ExecRGBLEDFlickerResult
from .exec_rgb_led_get_color_result import ExecRGBLEDGetColorResult
from .exec_rgb_led_get_state_result import ExecRGBLEDGetStateResult
from .exec_rgb_led_single_pulse_result import ExecRGBLEDSinglePulseResult
from .exec_rgb_led_stop_result import ExecRGBLEDStopResult
from .exec_servo_drive_custom_device_init_result import (
    ExecServoDriveCustomDeviceInitResult,
)
from .exec_servo_drive_extend_result import ExecServoDriveExtendResult
from .exec_servo_drive_extend_to_model_result import ExecServoDriveExtendToModelResult
from .exec_servo_drive_get_current_angle_result import (
    ExecServoDriveGetCurrentAngleResult,
)
from .exec_servo_drive_get_state_result import ExecServoDriveGetStateResult
from .exec_servo_drive_min_step_rotate_result import ExecServoDriveMinStepRotateResult
from .exec_servo_drive_rotate_result import ExecServoDriveRotateResult
from .exec_servo_drive_rotate_with_relative_speed_result import (
    ExecServoDriveRotateWithRelativeSpeedResult,
)
from .exec_servo_drive_set_position_to_mid_working_range_result import (
    ExecServoDriveSetPositionToMidWorkingRangeResult,
)
from .exec_servo_drive_stop_result import ExecServoDriveStopResult
from .exec_servo_drive_turn_by_duty_cycle_result import (
    ExecServoDriveTurnByDutyCycleResult,
)
from .exec_servo_drive_turn_by_pulse_result import ExecServoDriveTurnByPulseResult
from .exec_servo_drive_turn_result import ExecServoDriveTurnResult
from .exec_servo_drive_turn_with_relative_speed_result import (
    ExecServoDriveTurnWithRelativeSpeedResult,
)
from .executor_extend_result import ExecutorExtendResult
from .executor_state_result import ExecutorStateResult
from .init_sdk_result import InitSDKResult
from .link_led_to_controller_result import LinkLedToControllerResult
from .link_pwm_to_controller_result import LinkPWMToControllerResult
from .link_r_servodrive_to_controller_result import LinkRServodriveToControllerResult
from .link_servodrive_to_controller_result import LinkServodriveToControllerResult
from .link_voltage_sensor_to_controller_result import LinkVoltageSensorToControllerResult
from .sensor_extend_result import SensorExtendResult
from .sensor_voltage_sensor_current_result import SensorVoltageSensorCurrentResult
from .sensor_voltage_sensor_custom_device_init_result import (
    SensorVoltageSensorCustomDeviceInitResult,
)
from .sensor_voltage_sensor_extend_result import SensorVoltageSensorExtendResult
from .sensor_voltage_sensor_extend_to_model_result import (
    SensorVoltageSensorExtendToModelResult,
)
from .sensor_voltage_sensor_power_result import SensorVoltageSensorPowerResult
from .sensor_voltage_sensor_read_reg_bytes_result import (
    SensorVoltageSensorReadRegBytesResult,
)
from .sensor_voltage_sensor_sense_result import SensorVoltageSensorSenseResult
from .sensor_voltage_sensor_voltage_result import SensorVoltageSensorVoltageResult
from .sensor_voltage_sensor_voltage_shunt_result import (
    SensorVoltageSensorVoltageShuntResult,
)
from .sensor_voltage_sensor_write_reg_bytes_result import (
    SensorVoltageSensorWriteRegBytesResult,
)
from .sigmod_pwm_close_result import SigmodPWMCloseResult
from .sigmod_pwm_extend_result import SigmodPWMExtendResult
from .sigmod_pwm_extend_to_model_result import SigmodPWMExtendToModelResult
from .sigmod_pwm_get_freq_result import SigmodPWMGetFreqResult
from .sigmod_pwm_get_port_duty_cycle_result import SigmodPWMGetPortDutyCycleResult
from .sigmod_pwm_get_port_freq_result import SigmodPWMGetPortFreqResult
from .sigmod_pwm_get_resolution_result import SigmodPWMGetResolutionResult
from .sigmod_pwm_read_byte_result import SigmodPWMReadByteResult
from .sigmod_pwm_read_reg_bytes_result import SigmodPWMReadRegBytesResult
from .sigmod_pwm_reset_all_result import SigmodPWMResetAllResult
from .sigmod_pwm_reset_port_result import SigmodPWMResetPortResult
from .sigmod_pwm_set_freq_result import SigmodPWMSetFreqResult
from .sigmod_pwm_set_port_duty_cycle_result import SigmodPWMSetPortDutyCycleResult
from .sigmod_pwm_set_port_freq_result import SigmodPWMSetPortFreqResult
from .sigmod_pwm_write_byte_result import SigmodPWMWriteByteResult
from .sigmod_pwm_write_reg_bytes_result import SigmodPWMWriteRegBytesResult



import dataclasses


@dataclasses.dataclass(frozen=True)
class SigmodPWMGetPortDutyCycleResult:
    # Код ошибки. При успехе всегда 0
    error_code: int
    # on - Указатель на количество тактов до перевода выхода в
    #    состояние логической «1»
    on: int
    # off - Указатель на количество тактов до перевода выхода в
    #    состояние логического «0»
    off: int



import dataclasses


@dataclasses.dataclass(frozen=True)
class SigmodPWMReadRegBytesResult:
    # Код ошибки. При успехе всегда 0
    error_code: int
    # buf - Указатель на массив байт для чтения
    buf: bytes
    # readBytesLen - Указатель на количество записанных байтов
    read_bytes_len: int



import dataclasses


@dataclasses.dataclass(frozen=True)
class SensorVoltageSensorPowerResult:
    # Код ошибки. При успехе всегда 0
    error_code: int
    # power - Указатель на значение мощности (Ватт)
    power: float



import dataclasses


@dataclasses.dataclass(frozen=True)
class ExecServoDriveGetCurrentAngleResult:
    # Код ошибки. При успехе всегда 0
    error_code: int
    # angle - Указатель на код состояния сервопривода
    angle: int



import ctypes


def convert_python_bytes_to_c_ulonglong(value: bytes) -> ctypes.c_ulonglong:
    return ctypes.c_ulonglong(int.from_bytes(value, "little"))


def convert_c_ulonglong_to_python_bytes(
    value: ctypes.c_ulonglong,
) -> bytes:
    """
    If we don't know len

    :param value:
    :return:
    """
    # Extract the integer value from the ctypes.c_ulonglong object
    int_value = value.value
    # Convert the integer value to a bytes object
    bytes_value = int_value.to_bytes((int_value.bit_length() + 7) // 8, "big")
    return bytes_value


def convert_c_ulonglong_of_len_to_python_bytes(
    value: ctypes.c_ulonglong,
    read_bytes_len: ctypes.c_int,
) -> bytes:
    """
    As in docs

    :param value:
    :param read_bytes_len:
    :return:
    """
    return value.value.to_bytes(read_bytes_len.value, "little")
